#include <pspsdk.h> 
#include <global.h>
#include <pspnet_apctl.h>
#include <pspnet_inet.h>
#include <netinet/in.h>
#include <sys/socket.h> 
#include "sockhak.h"
#include <stdio.h>
#include <curl/curl.h>
#include <pspnet_resolver.h>
int sceNetInetInetAton(const char* host, struct in_addr* addr);

//NOTE: kernel mode module flag and kernel mode thread are both required
PSP_MODULE_INFO("curl", 0x1000, 1, 1); /* 0x1000 REQUIRED!!! */
PSP_MAIN_THREAD_ATTR(0); /* 0 REQUIRED!!! */
PSP_MAIN_THREAD_STACK_SIZE_KB(32); /* smaller stack for kernel thread */

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common)
{
    sceKernelExitGame();

    return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
    int cbid;

    cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
    sceKernelRegisterExitCallback(cbid);

    sceKernelSleepThreadCB();
    return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
    int thid = 0;

    thid = sceKernelCreateThread("update_thread", CallbackThread, 0x10, 0xFA0, 0, 0);
    if(thid >= 0)
    {
        sceKernelStartThread(thid, 0, 0);
    }

    return thid;
}

PspDebugStackTrace st[3];

/* Example custom exception handler */
void MyExceptionHandler(PspDebugRegBlock *regs)
{
    /* Do normal initial dump, setup screen etc */
    pspDebugScreenInit();
   
    /* I always felt BSODs were more interesting that white on black */
    pspDebugScreenSetBackColor(0x00FF0000);
    pspDebugScreenSetTextColor(0xFFFFFFFF);
    pspDebugScreenClear();

    pspDebugScreenPrintf("Exception Details:\n");
    pspDebugDumpException(regs);
    pspDebugScreenPrintf("\nStack Trace:\n");

    int size = pspDebugGetStackTrace2(regs, st, 3);
    int a;
    for (a = 0; a < size; a++)
    {
        printf("%i)  %x | %x\n", a, st[a].call_addr, st[a].func_addr);
    }
    sceKernelDelayThread(10*1000*1000); // 10sec
}

int user_main(SceSize argc, void* argv);

int main(void)
{
    // Kernel mode thread
    pspDebugScreenInit();
    pspDebugInstallErrorHandler(MyExceptionHandler);
	SetupCallbacks();
int err = pspSdkLoadInetModules(); 
   if (err != 0) {
      pspDebugScreenPrintf("pspSdkLoadInetModules failed with %x\n", err);
      sceKernelDelayThread(5*1000000); // 5 sec to read error
      return 1;
   } 
	

   // create user thread, tweek stack size here if necessary
   SceUID thid = sceKernelCreateThread("User Mode Thread", user_main,
      0x11, // default priority
      256 * 1024, // stack size (256KB is regular default)
      PSP_THREAD_ATTR_USER, NULL);
   
   // start user thread, then wait for it to do everything else
   sceKernelStartThread(thid, 0, NULL);
   sceKernelWaitThreadEnd(thid, NULL); 
   return 0;
}

char resolverBuffer[1024];
int resolverId;

size_t write_data(void *ptr, size_t size, size_t nmemb, void *stream)
{
	int a;
	char* buf = (char*)ptr;
	
//	pspDebugScreenPrintf("Received %i bytes?\n", size*nmemb);
	for (a = 0; a < (size * nmemb); a++)
	{
		pspDebugScreenPrintf("%c", buf[a]);
	}
	return nmemb;
//  int written = fwrite(ptr, size, nmemb, (FILE *)stream);
//  return written;
}


int user_main(SceSize argc, void* argv) 
{

   // init wlan
   int err = pspSdkInetInit();
   if (err != 0) {
      pspDebugScreenPrintf("pspSdkInetInit failed: %i", err);
//      return;
   }

   // use connection 1
   pspDebugScreenPrintf("using first connection\n");
   err = sceNetApctlConnect(1);
   if (err != 0) {
      pspDebugScreenPrintf("sceNetApctlConnect failed: %i", err);
//      return;
   }

   // WLAN is initialized when PSP IP address is available
   pspDebugScreenPrintf("init WLAN and getting IP address");
   char pspIPAddr[32];
   while (1) {
      if (sceNetApctlGetInfo(8, pspIPAddr) == 0) break;
      pspDebugScreenPrintf(".");
      sceKernelDelayThread(1000 * 1000);  // wait a second
   }
   pspDebugScreenPrintf("\nPSP IP address: %s\n", pspIPAddr);


   err = sceNetResolverCreate(&resolverId, resolverBuffer, sizeof(resolverBuffer));
   if (err != 0) {
      pspDebugScreenPrintf("sceNetResolverCreate failed: %i", err);
      sceKernelDelayThread(1000*1000*4);
//      return -1;
   } 
   else
   {
   	pspDebugScreenPrintf("Created resolver\n");
   }


  CURL *curl;
  CURLcode res;

    // Kernel mode thread
//    pspDebugScreenInit();
pspDebugScreenPrintf("GONNA CURL!\n");

sceKernelDelayThread(1000*1000*1);

  curl = curl_easy_init();
   if(curl) {
	//set url
     curl_easy_setopt(curl, CURLOPT_URL, "http://localhost.geek.nz");
	//no prog meter (i think this is broke)
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
	//shut up completely
	curl_easy_setopt(curl, CURLOPT_MUTE, 1);
	
	//send data here
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data); 

     res = curl_easy_perform(curl);

     // always cleanup 
     curl_easy_cleanup(curl);
   }
pspDebugScreenPrintf("CURLED! res%i\n", res);



/*   err = sceNetResolverCreate(&resolverId, resolverBuffer, sizeof(resolverBuffer));
   if (err != 0) {
      pspDebugScreenPrintf("sceNetResolverCreate failed: %i", err);
      sceKernelDelayThread(1000*1000*4);
//      return -1;
   } 
   // resolve host
   //pspDebugScreenPrintf("resolving host...\n"); //removed
   const char *host = "localhost.geek.nz";
   int port = 80;
   struct sockaddr_in addrTo;
   addrTo.sin_family = AF_INET;
   addrTo.sin_port = htons(port);
   err = sceNetInetInetAton(host, &addrTo.sin_addr);
   if (err == 0) {
      err = sceNetResolverStartNtoA(resolverId, host, &addrTo.sin_addr, 2, 3);
      if (err != 0) {
         pspDebugScreenPrintf("sceNetResolverStartNtoA failed: %i\n", err);
//         return;
      }
   }
   pspDebugScreenPrintf("Done\n");
*/
sceKernelDelayThread(1000*1000*4);
  return 0;
}
 
